#!/bin/bash

help() {
	echo -e "\e[91m# Usage: ./mt_agent_container.sh [start|stop|help]\e[0m"
	echo -e "\e[91m# start - Start the script\e[0m"
	echo -e "\e[91m# stop  - Stop and remove containers\e[0m"
	echo -e "\e[91m# help  - Display this help message\e[0m"
}

start() {
container_manager=""

# Check if podman is available
if   command -v podman &> /dev/null; then
     container_manager="podman"

# Check if docker is available
elif command -v docker &> /dev/null; then
     container_manager="docker"
else
     echo -e "\e[91m# Podman or Docker not found\e[0m"
     echo -e "\e[91m# Install a container manager\e[0m"
     exit 1
fi
     echo -e "\e[91m# Using container manager: $container_manager\e[0m"

# Check if metatester64.exe is available
if [ ! -e metatester64.exe ]; then
     echo -e "\e[91m# metatester64.exe at $(pwd) not found\e[0m"
     echo -e "\e[91m# Put metatester64.exe in $(pwd)\e[0m"
     exit 1
fi

# Create entrypoint file
echo -e "\e[91m# Create entrypoint.sh in $(pwd)\e[0m"
cat > entrypoint.sh << EOF;
#!/bin/sh
tmux new -d;
tmux send-keys -t 0:0 'tmux new -d wine cmd /K ''"metatester64.exe /install /address:0.0.0.0:\$port /password:\$password && net start MetaTester-1"''' Enter;
tmux send-keys -t 0:0 'while true; do if [ -z "\$(pgrep metatester64.exe)" ]; then tmux send-keys -t 1:0 ''"net start MetaTester-1 "'' Enter; fi; sleep 1; done' Enter;
tmux a;
EOF

# Create Docker file
echo -e "\e[91m# Create Dockerfile in $(pwd)\e[0m"
cat > Dockerfile << EOF;
FROM alpine:latest
RUN apk update && apk add wine tmux
COPY metatester64.exe /
COPY entrypoint.sh /
RUN chmod +x entrypoint.sh
ENV port=2000;
ENV password=MetaTester
ENV WINEDEBUG=-all
ENV WINEPREFIX=/wine
ENTRYPOINT ["/entrypoint.sh"]
EOF

#Stop and remove containers, remove image
stop;

#Build image
echo -e "\e[91m# Build image metatester_agent\e[0m"
$container_manager build -t metatester_agent .

#Launching containers: one container per core.
echo -e "\e[91m# Launching containers: one container per core.\e[0m"
for p in $(seq -w 00 $(($(nproc) - 1))); do
	$container_manager run -dit --restart always --network=host --privileged=true --cpuset-cpus=$p -e port=20$p --name metatester_agent_cpu_$p metatester_agent;
done

echo -e "\e[91m# Complete #\e[0m"
}

stop() {

container_manager=""

# Check if podman is available
if   command -v podman &> /dev/null; then
     container_manager="podman"

# Check if docker is available
elif command -v docker &> /dev/null; then
     container_manager="docker"
else
     echo -e "\e[91m# Podman or Docker not found\e[0m"
     echo -e "\e[91m# Install a container manager\e[0m"
     exit 1
fi
     echo -e "\e[91m# Using container manager: $container_manager\e[0m"

if [ ! -n "$($container_manager ps -qa --filter name=metatester_agent)" ] && [ ! -n "$($container_manager ps -qa --filter name=metatester_agent)" ]; then
     echo -e "\e[91m# There is nothing to delete or stop\e[0m"
else

    #Checking for running containers with the "metatester_agent" name, stopping and removing them.
if [ -n "$($container_manager ps -qa --filter name=metatester_agent)" ]; then
    echo -e "\e[91m# Stopping containers with metatester_agent\e[0m"
    $container_manager stop $($container_manager ps -qa --filter name=metatester_agent);
    #We are waiting for the removal of containers if they were launched with the --rm flag.
    sleep 10;
fi
    #If containers were launched without the '--rm' flag and were not removed after being stopped, check their presence and delete them.
if [ -n "$($container_manager ps -qa --filter name=metatester_agent)" ]; then
    echo -e "\e[91m# Remove containers with metatester_agent\e[0m"
    $container_manager rm $($container_manager ps -qa --filter name=metatester_agent);
fi
    #Checking for the presence of the "metatester_agent" image and cleaning it up.
if [ -n "$($container_manager images -q --filter reference=metatester_agent)" ]; then
    echo -e "\e[91m# Remove image with metatester_agent\e[0m"
    $container_manager image rm metatester_agent;
fi
fi
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    help)
        help
        ;;
    *)
        echo -e "\e[91m# Error: Unknown command. Use 'start', 'stop', or 'help'\e[0m"
        exit 1
        ;;
esac

exit 0
